@@ -8,6 +8,7 @@ import (
 	"bufio"
 	"context"
 	"crypto/tls"
+	"errors"
 	"fmt"
 	"io"
 	"log"
@@ -18,8 +19,8 @@ import (
 	"time"
 
 	"github.com/ProtonMail/gopenpgp/v2/crypto" // MIT License
-	"github.com/mattn/go-xmpp"                 // BSD-3-Clause
 	"github.com/pborman/getopt/v2"             // BSD-3-Clause
+	"github.com/xmppo/go-xmpp"                 // BSD-3-Clause
 )
 
 type configuration struct {
@@ -30,6 +31,17 @@ type configuration struct {
 	alias    string
 }
 
+func closeAndExit(client *xmpp.Client, cancel context.CancelFunc, err error) {
+	// Wait for a short time as some messages are not delivered by the server
+	// if the connection is closed immediately after sending a message.
+	time.Sleep(defaultSleepTime * time.Millisecond)
+	cancel()
+	client.Close()
+	if err != nil {
+		log.Fatal(err)
+	}
+}
+
 func readMessage(messageFilePath string) (string, error) {
 	var (
 		output string
@@ -223,6 +235,7 @@ func main() {
 	tlsConfig.ServerName = user[strings.Index(user, "@")+1:]
 	tlsConfig.NextProtos = append(tlsConfig.NextProtos, "xmpp-client")
 	tlsConfig.InsecureSkipVerify = *flagSkipVerify
+	tlsConfig.Renegotiation = tls.RenegotiateNever
 	switch *flagTLSMinVersion {
 	case defaultTLS10:
 		tlsConfig.MinVersion = tls.VersionTLS10
@@ -249,7 +262,7 @@ func main() {
 		// an unencrypted connection is established. As StartTLS is
 		// set when NoTLS is set go-sendxmpp won't use unencrypted
 		// client-to-server connections.
-		// See https://pkg.go.dev/github.com/mattn/go-xmpp#Options
+		// See https://pkg.go.dev/github.com/xmppo/go-xmpp#Options
 		NoTLS:     !*flagDirectTLS,
 		StartTLS:  !*flagDirectTLS,
 		Debug:     *flagDebug,
@@ -323,7 +336,7 @@ func main() {
 	for i, recipient := range recipients {
 		validatedJid, err := MarshalJID(recipient.Jid)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		recipients[i].Jid = validatedJid
 	}
@@ -332,58 +345,61 @@ func main() {
 	case *flagOxGenPrivKeyX25519:
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		err = oxGenPrivKey(validatedOwnJid, client, iqc, *flagOxPassphrase, "x25519")
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		os.Exit(0)
 	case *flagOxGenPrivKeyRSA:
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		err = oxGenPrivKey(validatedOwnJid, client, iqc, *flagOxPassphrase, "rsa")
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		os.Exit(0)
 	case *flagOxImportPrivKey != "":
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		err = oxImportPrivKey(validatedOwnJid, *flagOxImportPrivKey,
 			client, iqc)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		os.Exit(0)
 	case *flagOxDeleteNodes:
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		err = oxDeleteNodes(validatedOwnJid, client, iqc)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		os.Exit(0)
 	case *flagOx:
 		validatedOwnJid, err := MarshalJID(user)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		oxPrivKey, err = oxGetPrivKey(validatedOwnJid, *flagOxPassphrase)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 	}
 
 	if *flagHTTPUpload != "" {
-		message = httpUpload(client, iqc, tlsConfig.ServerName,
-			*flagHTTPUpload)
+		message, err = httpUpload(client, iqc, tlsConfig.ServerName,
+			*flagHTTPUpload, timeout)
+		if err != nil {
+			closeAndExit(client, cancel, err)
+		}
 	}
 
 	if *flagOOBFile != "" {
@@ -392,7 +408,7 @@ func main() {
 		// Check if the URI is valid.
 		uri, err := validURI(message)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 		message = uri.String()
 	}
@@ -415,7 +431,7 @@ func main() {
 				_, err = client.JoinMUCNoHistory(recipient.Jid, alias)
 			}
 			if err != nil {
-				log.Fatal(err)
+				closeAndExit(client, cancel, err)
 			}
 		}
 	}
@@ -427,7 +443,7 @@ func main() {
 		// Send raw XML
 		_, err = client.SendOrg(message)
 		if err != nil {
-			log.Fatal(err)
+			closeAndExit(client, cancel, err)
 		}
 	case *flagInteractive:
 		// Send in endless loop (for usage with e.g. "tail -f").
@@ -446,7 +462,7 @@ func main() {
 			message, err = reader.ReadString('\n')
 			message = strings.TrimSuffix(message, "\n")
 			if err != nil {
-				log.Fatal("failed to read from stdin")
+				closeAndExit(client, cancel, errors.New("failed to read from stdin"))
 			}
 
 			// Remove invalid code points.
@@ -469,7 +485,7 @@ func main() {
 					}
 					_, err = client.SendOrg(oxMessage)
 					if err != nil {
-						log.Fatal(err)
+						closeAndExit(client, cancel, err)
 					}
 				default:
 					_, err = client.Send(xmpp.Chat{
@@ -477,7 +493,7 @@ func main() {
 						Type:   msgType, Text: message,
 					})
 					if err != nil {
-						log.Fatal(err)
+						closeAndExit(client, cancel, err)
 					}
 				}
 			}
@@ -588,7 +604,7 @@ func main() {
 				}
 				_, err = client.SendOrg(oxMessage)
 				if err != nil {
-					log.Fatal(err)
+					closeAndExit(client, cancel, err)
 				}
 			default:
 				_, err = client.Send(xmpp.Chat{
@@ -596,15 +612,10 @@ func main() {
 					Type:   msgType, Text: message,
 				})
 				if err != nil {
-					log.Fatal(err)
+					closeAndExit(client, cancel, err)
 				}
 			}
 		}
 	}
-
-	// Wait for a short time as some messages are not delivered by the server
-	// if the connection is closed immediately after sending a message.
-	time.Sleep(defaultSleepTime * time.Millisecond)
-	cancel()
-	client.Close()
+	closeAndExit(client, cancel, nil)
 }
