@@ -16,7 +16,7 @@ import (
 
 	"github.com/ProtonMail/gopenpgp/v2/crypto" // MIT License
 	"github.com/beevik/etree"                  // BSD-2-clause
-	"github.com/mattn/go-xmpp"                // BSD-3-Clause
+	"github.com/xmppo/go-xmpp"                 // BSD-3-Clause
 )
 
 func oxDeleteNodes(jid string, client *xmpp.Client, iqc chan xmpp.IQ) error {
